package envoy.client;

import java.io.InputStream;
import java.io.ObjectInputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import envoy.client.util.EnvoyLog;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>Receiver.java</strong><br>
 * Created: <strong>30.12.2019</strong><br>
 * 
 * @author Kai S. K. Engelbart
 * @since Envoy v0.3-alpha
 */
public class Receiver implements Runnable {

	private InputStream				in;
	private Map<Class<?>, ObjectProcessor<?>>	processors	= new HashMap<>();

	private static final Logger logger = EnvoyLog.getLogger(Receiver.class.getSimpleName());

	public Receiver(InputStream in) { this.in = in; }

	@SuppressWarnings("unchecked")
	@Override
	public void run() {
		try (ObjectInputStream oin = new ObjectInputStream(in)) {
			while (true) {
				Object obj = oin.readObject();
				logger.finest("Received object " + obj);

				// Get appropriate processor
				ObjectProcessor processor = processors.get(obj.getClass());
				if (processor == null)
					logger.severe(String.format("The received object has the class %s for which no processor is defined.", obj.getClass()));
				else
					processor.process(obj);
			}
		} catch(Exception e) {
			logger.log(Level.SEVERE, "Error on receiver thread", e);
		}
	}

	public <T> void registerProcessor(Class<T> processorClass, ObjectProcessor<T> processor) { processors.put(processorClass, processor); }
}